//
//  ChargeRecordMetricsTableView.swift
//  USB Meter
//
//  Created by Bogdan Timofte on 29/03/2026.
//  Co-authored-by: GPT-5.3-Codex.
//  Copyright © 2026 Bogdan Timofte. All rights reserved.
//

import SwiftUI

struct ChargeRecordMetricsTableView: View {
    let labels: [String]
    let values: [String]

    var body: some View {
        HStack(alignment: .top) {
            VStack(alignment: .leading, spacing: 10) {
                ForEach(labels, id: \.self) { label in
                    Text(label)
                }
            }
            Spacer()
            VStack(alignment: .trailing, spacing: 10) {
                ForEach(Array(values.enumerated()), id: \.offset) { _, value in
                    Text(value)
                }
            }
            .monospacedDigit()
        }
        .font(.footnote.weight(.semibold))
    }
}