//
//  Float.swift
//  USB Meter
//
//  Created by Bogdan Timofte on 08/03/2020.
//  Copyright © 2020 Bogdan Timofte. All rights reserved.
//

import Foundation

extension Float {
    var uInt8Value: UInt8 {
        return UInt8(self)
    }
    
    var doubleValue: Double {
        return Double(self)
    }
}
