//
//  SidebarHelpReason.swift
//  USB Meter
//

import SwiftUI

enum SidebarHelpReason: String {
    case bluetoothPermission
    case cloudSyncUnavailable
    case noDevicesDetected

    var tint: Color {
        switch self {
        case .bluetoothPermission:
            return .orange
        case .cloudSyncUnavailable:
            return .indigo
        case .noDevicesDetected:
            return .yellow
        }
    }

    var symbol: String {
        switch self {
        case .bluetoothPermission:
            return "bolt.horizontal.circle.fill"
        case .cloudSyncUnavailable:
            return "icloud.slash.fill"
        case .noDevicesDetected:
            return "magnifyingglass.circle.fill"
        }
    }

    var badgeTitle: String {
        switch self {
        case .bluetoothPermission:
            return "Required"
        case .cloudSyncUnavailable:
            return "Sync Off"
        case .noDevicesDetected:
            return "Suggested"
        }
    }
}